@@ -39,9 +39,13 @@
 
 #include <algorithm>
 #include <cctype>
+#include <climits>
 #include <cstdlib>
+#include <functional>
 #include <iterator>
 #include <locale>
+#include <random>
+
 
 namespace rak {
 
@@ -139,8 +143,8 @@ public:
     return *this;
   }
 
-  bool operator == (__UNUSED const split_iterator_t& itr) const { return m_pos == m_seq->end(); }
-  bool operator != (__UNUSED const split_iterator_t& itr) const { return m_pos != m_seq->end(); }
+  bool operator == (const split_iterator_t&) const { return m_pos == m_seq->end(); }
+  bool operator != (const split_iterator_t&) const { return m_pos != m_seq->end(); }
 
 private:
   const Sequence* m_seq;
@@ -157,7 +161,7 @@ split_iterator(const Sequence& seq, typename Sequence::value_type delim) {
 
 template <typename Sequence>
 inline split_iterator_t<Sequence>
-split_iterator(__UNUSED const Sequence& seq) {
+split_iterator(const Sequence&) {
   return split_iterator_t<Sequence>();
 }
 
@@ -228,8 +232,17 @@ copy_escape_html(InputIterator first1, InputIterator last1, OutputIterator first
   return first2;
 }
 
+template <typename Iterator>
+inline std::string
+copy_escape_html(Iterator first, Iterator last) {
+  std::string dest;
+  copy_escape_html(first, last, std::back_inserter(dest));
+
+  return dest;
+}
+
 template <typename Sequence>
-Sequence
+inline Sequence
 copy_escape_html(const Sequence& src) {
   Sequence dest;
   copy_escape_html(src.begin(), src.end(), std::back_inserter(dest));
@@ -237,6 +250,15 @@ copy_escape_html(const Sequence& src) {
   return dest;
 }
 
+template <typename Sequence>
+inline std::string
+copy_escape_html_str(const Sequence& src) {
+  std::string dest;
+  copy_escape_html(src.begin(), src.end(), std::back_inserter(dest));
+
+  return dest;
+}
+
 // Consider support for larger than char type.
 template <typename InputIterator, typename OutputIterator> 
 OutputIterator
@@ -282,17 +304,124 @@ transform_hex(Iterator first, Iterator last) {
   return dest;
 }
 
+template <typename Sequence>
+inline std::string
+transform_hex_str(const Sequence& seq) {
+  std::string dest;
+  transform_hex(seq.begin(), seq.end(), std::back_inserter(dest));
+
+  return dest;
+}
+
 template <typename Sequence>
 Sequence
 generate_random(size_t length) {
+  std::random_device rd;
+  std::mt19937 mt(rd());
+  using bytes_randomizer = std::independent_bits_engine<std::mt19937, CHAR_BIT, uint8_t>;
+  bytes_randomizer bytes(mt);
   Sequence s;
   s.reserve(length);
-
-  std::generate_n(std::back_inserter(s), length, &::random);
-
+  std::generate_n(std::back_inserter(s), length, std::ref(bytes));
   return s;
 }
 
+template <typename Iterator>
+inline bool
+is_all_alpha(Iterator first, Iterator last) {
+  while (first != last)
+    if (!std::isalpha(*first++, std::locale::classic()))
+      return false;
+
+  return true;
+}
+
+template <typename Sequence>
+inline bool
+is_all_alpha(const Sequence& src) {
+  return is_all_alpha(src.begin(), src.end());
+}
+
+template <typename Iterator>
+inline bool
+is_all_alnum(Iterator first, Iterator last) {
+  while (first != last)
+    if (!std::isalnum(*first++, std::locale::classic()))
+      return false;
+
+  return true;
+}
+
+template <typename Sequence>
+inline bool
+is_all_alnum(const Sequence& src) {
+  return is_all_alnum(src.begin(), src.end());
+}
+
+template <typename Iterator>
+inline bool
+is_all_name(Iterator first, Iterator last) {
+  while (first != last) {
+    if (!std::isalnum(*first, std::locale::classic()) && *first != '_')
+      return false;
+
+    first++;
+  }
+
+  return true;
+}
+
+template <typename Sequence>
+inline bool
+is_all_name(const Sequence& src) {
+  return is_all_name(src.begin(), src.end());
+}
+
+template <typename Iterator>
+std::string
+sanitize(Iterator first, Iterator last) {
+  std::string dest;
+  for (; first != last; ++first) {
+    if (std::isprint(*first) && *first != '\r' && *first != '\n' && *first != '\t')
+      dest += *first;
+    else
+      dest += " ";
+  }
+
+  return dest;
+}
+
+template <typename Sequence>
+std::string
+sanitize(const Sequence& src) {
+    return trim(sanitize(src.begin(), src.end()));
+}
+
+template <typename Iterator>
+std::string striptags(Iterator first, Iterator last) {
+  bool copychar = true;
+  std::string dest;
+
+  for (; first != last; ++first) {
+    if (std::isprint(*first) && *first == '<') {
+      copychar = false;
+    } else if (std::isprint(*first) && *first == '>') {
+      copychar = true;
+      continue;
+    }
+
+    if (copychar)
+      dest += *first;
+  }
+
+  return dest;
+}
+
+template <typename Sequence>
+std::string striptags(const Sequence& src) {
+    return striptags(src.begin(), src.end());
+}
+
 }
 
 #endif
