@@ -1,44 +1,10 @@
-// rTorrent - BitTorrent client
-// Copyright (C) 2005-2011, Jari Sundell
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// 
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-//
-// In addition, as a special exception, the copyright holders give
-// permission to link the code of portions of this program with the
-// OpenSSL library under certain conditions as described in each
-// individual source file, and distribute linked combinations
-// including the two.
-//
-// You must obey the GNU General Public License in all respects for
-// all of the code used other than OpenSSL.  If you modify file(s)
-// with this exception, you may extend this exception to your version
-// of the file(s), but you are not obligated to do so.  If you do not
-// wish to do so, delete this exception statement from your version.
-// If you delete this exception statement from all source files in the
-// program, then also delete it here.
-//
-// Contact:  Jari Sundell <jaris@ifi.uio.no>
-//
-//           Skomakerveien 33
-//           3185 Skoppum, NORWAY
-
 #include "config.h"
 
-#include <functional>
-#include <unistd.h>
+#include <cassert>
 #include <cstdio>
+#include <functional>
+#include <netdb.h>
+#include <unistd.h>
 #include <rak/file_stat.h>
 #include <rak/error_number.h>
 #include <rak/path.h>
@@ -47,14 +13,14 @@
 #include <rak/regex.h>
 #include <torrent/rate.h>
 #include <torrent/throttle.h>
-#include <torrent/tracker.h>
-#include <torrent/tracker_controller.h>
-#include <torrent/tracker_list.h>
+#include <torrent/tracker/tracker.h>
 #include <torrent/connection_manager.h>
 #include <torrent/data/download_data.h>
 #include <torrent/data/file.h>
 #include <torrent/data/file_list.h>
 #include <torrent/download/resource_manager.h>
+#include <torrent/net/resolver.h>
+#include <torrent/net/types.h>
 #include <torrent/peer/connection_list.h>
 #include <torrent/peer/peer_list.h>
 #include <torrent/utils/log.h>
@@ -104,7 +70,7 @@ apply_d_change_link(core::Download* download, const torrent::Object::list_type&
   const std::string& type    = (itr++)->as_string();
   const std::string& prefix  = (itr++)->as_string();
   const std::string& postfix = (itr++)->as_string();
-  
+
   if (type.empty())
     throw torrent::input_error("Invalid arguments.");
 
@@ -319,21 +285,6 @@ retrieve_d_bitfield(core::Download* download) {
   return torrent::Object(rak::transform_hex(bitField->begin(), bitField->end()));
 }
 
-struct call_add_d_peer_t {
-  call_add_d_peer_t(core::Download* d, int port) : m_download(d), m_port(port) { }
-
-  void operator() (const sockaddr* sa, int err) {
-    if (sa == NULL) {
-      lt_log_print(torrent::LOG_TORRENT_WARN, "could not resolve hostname for added peer");
-    } else {
-      m_download->download()->add_peer(sa, m_port);
-    }
-  }
-
-  core::Download* m_download;
-  int m_port;
-};
-
 void
 apply_d_add_peer(core::Download* download, const std::string& arg) {
   int port, ret;
@@ -356,7 +307,17 @@ apply_d_add_peer(core::Download* download, const std::string& arg) {
   if (port < 1 || port > 65535)
     throw torrent::input_error("Invalid port number.");
 
-  torrent::connection_manager()->resolver()(host, (int)rak::socket_address::pf_unspec, SOCK_STREAM, call_add_d_peer_t(download, port));
+  assert(std::this_thread::get_id() == torrent::main_thread::thread()->thread_id());
+
+  // Currently discarding SOCK_STREAM.
+  torrent::this_thread::resolver()->resolve_preferred(NULL, host, AF_UNSPEC, AF_INET, [download, port](torrent::c_sa_shared_ptr sa, int err) {
+      if (sa == nullptr) {
+        lt_log_print(torrent::LOG_TORRENT_WARN, "could not resolve hostname for added peer: %s", gai_strerror(err));
+        return;
+      }
+
+      download->download()->add_peer(sa.get(), port);
+  });
 }
 
 torrent::Object
@@ -410,7 +371,7 @@ f_multicall(core::Download* download, const torrent::Object::list_type& args) {
 
     for (torrent::Object::list_const_iterator cItr = ++args.begin(); cItr != args.end(); cItr++) {
       const std::string& cmd = cItr->as_string();
-      row.push_back(rpc::parse_command(rpc::make_target(*itr), cmd.c_str(), cmd.c_str() + cmd.size()).first);
+      row.push_back(rpc::parse_command(rpc::make_target(itr->get()), cmd.c_str(), cmd.c_str() + cmd.size()).first);
     }
   }
 
@@ -427,21 +388,24 @@ t_multicall(core::Download* download, const torrent::Object::list_type& args) {
 
   // Add some pre-parsing of the commands, so we don't spend time
   // parsing and searching command map for every single call.
-  torrent::Object             resultRaw = torrent::Object::create_list();
-  torrent::Object::list_type& result = resultRaw.as_list();
+  torrent::Object             result_raw = torrent::Object::create_list();
+  torrent::Object::list_type& result     = result_raw.as_list();
 
-  for (int itr = 0, last = download->tracker_list()->size(); itr != last; itr++) {
-    torrent::Object::list_type& row = result.insert(result.end(), torrent::Object::create_list())->as_list();
+  for (uint32_t idx = 0, last = download->tracker_list_size(); idx < last; idx++) {
+    auto& row = result.insert(result.end(), torrent::Object::create_list())->as_list();
+    auto  tracker = download->tracker_controller().at(idx);
+
+    if (!tracker.is_valid())
+      continue;
 
     for (torrent::Object::list_const_iterator cItr = ++args.begin(); cItr != args.end(); cItr++) {
       const std::string& cmd = cItr->as_string();
-      torrent::Tracker* t = download->tracker_list()->at(itr);
 
-      row.push_back(rpc::parse_command(rpc::make_target(t), cmd.c_str(), cmd.c_str() + cmd.size()).first);
+      row.push_back(rpc::parse_command(rpc::make_target(&tracker), cmd.c_str(), cmd.c_str() + cmd.size()).first);
     }
   }
 
-  return resultRaw;
+  return result_raw;
 }
 
 torrent::Object
@@ -522,7 +486,8 @@ download_tracker_insert(core::Download* download, const torrent::Object::list_ty
   if (group < 0 || group > 32)
     throw torrent::input_error("Tracker group number invalid.");
 
-  download->download()->tracker_list()->insert_url(group, args.back().as_string(), true);
+  download->tracker_controller().add_extra_tracker(group, args.back().as_string());
+
   return torrent::Object();
 }
 
@@ -626,7 +591,6 @@ d_list_remove(core::Download* download, const torrent::Object& rawArgs, const ch
 #define CMD2_BIND_CL std::bind(&core::Download::connection_list, std::placeholders::_1)
 #define CMD2_BIND_FL std::bind(&core::Download::file_list, std::placeholders::_1)
 #define CMD2_BIND_PL std::bind(&core::Download::c_peer_list, std::placeholders::_1)
-#define CMD2_BIND_TL std::bind(&core::Download::tracker_list, std::placeholders::_1)
 #define CMD2_BIND_TC std::bind(&core::Download::tracker_controller, std::placeholders::_1)
 
 #define CMD2_BIND_INFO std::bind(&core::Download::info, std::placeholders::_1)
@@ -871,17 +835,20 @@ initialize_command_download() {
   CMD2_DL         ("d.wanted_chunks",    CMD2_ON_DATA(wanted_chunks));
 
   // Do not exposre d.tracker_announce.force to regular users.
-  CMD2_DL_V       ("d.tracker_announce",       std::bind(&torrent::Download::manual_request, CMD2_BIND_DL, false)); 
-  CMD2_DL_V       ("d.tracker_announce.force", std::bind(&torrent::Download::manual_request, CMD2_BIND_DL, true)); 
+  CMD2_DL_V       ("d.tracker_announce",       std::bind(&torrent::Download::manual_request, CMD2_BIND_DL, false));
+  CMD2_DL_V       ("d.tracker_announce.force", std::bind(&torrent::Download::manual_request, CMD2_BIND_DL, true));
 
-  CMD2_DL         ("d.tracker_numwant",      std::bind(&torrent::TrackerList::numwant, CMD2_BIND_TL));
-  CMD2_DL_VALUE_V ("d.tracker_numwant.set",  std::bind(&torrent::TrackerList::set_numwant, CMD2_BIND_TL, std::placeholders::_2));
+  CMD2_DL         ("d.tracker_numwant",      std::bind(&torrent::tracker::TrackerControllerWrapper::numwant, CMD2_BIND_TC));
+  CMD2_DL_VALUE_V ("d.tracker_numwant.set",  std::bind(&torrent::tracker::TrackerControllerWrapper::set_numwant, CMD2_BIND_TC, std::placeholders::_2));
   // TODO: Deprecate 'd.tracker_focus'.
   CMD2_DL         ("d.tracker_focus",        std::bind(&core::Download::tracker_list_size, std::placeholders::_1));
   CMD2_DL         ("d.tracker_size",         std::bind(&core::Download::tracker_list_size, std::placeholders::_1));
 
-  CMD2_DL_LIST    ("d.tracker.insert",       std::bind(&download_tracker_insert, std::placeholders::_1, std::placeholders::_2));
-  CMD2_DL_VALUE_V ("d.tracker.send_scrape",  std::bind(&torrent::TrackerController::scrape_request, CMD2_BIND_TC, std::placeholders::_2));
+  CMD2_DL         ("d.tracker.has_active",            std::bind(&torrent::tracker::TrackerControllerWrapper::has_active_trackers, CMD2_BIND_TC));
+  CMD2_DL         ("d.tracker.has_active_not_scrape", std::bind(&torrent::tracker::TrackerControllerWrapper::has_active_trackers_not_scrape, CMD2_BIND_TC));
+  CMD2_DL         ("d.tracker.has_usable",            std::bind(&torrent::tracker::TrackerControllerWrapper::has_usable_trackers, CMD2_BIND_TC));
+  CMD2_DL_LIST    ("d.tracker.insert",                std::bind(&download_tracker_insert, std::placeholders::_1, std::placeholders::_2));
+  CMD2_DL_VALUE_V ("d.tracker.send_scrape",           [](auto download, uint64_t arg) { download->tracker_controller().scrape_request(arg); });
 
   CMD2_DL         ("d.directory",          CMD2_ON_FL(root_dir));
   CMD2_DL_STRING_V("d.directory.set",      std::bind(&apply_d_directory, std::placeholders::_1, std::placeholders::_2));
