@@ -61,7 +61,6 @@ public:
 
   using base_type::push_back;
 
-  TextElementList() : m_column(0), m_columnWidth(0) {}
   virtual ~TextElementList() { clear(); }
 
   void                clear();
@@ -74,8 +73,8 @@ public:
   virtual extent_type max_length();
 
 private:
-  unsigned int        m_column;
-  extent_type*        m_columnWidth;
+  unsigned int        m_column{};
+  extent_type*        m_columnWidth{};
 };
 
 }
