@@ -50,8 +50,7 @@ class ElementBase {
 public:
   typedef std::function<void ()> slot_type;
 
-  ElementBase() : m_frame(NULL), m_focus(false) {}
-  virtual ~ElementBase() {}
+  virtual ~ElementBase() = default;
 
   bool                is_active() const { return m_frame != NULL; }
 
@@ -65,8 +64,8 @@ public:
   void                mark_dirty();
 
 protected:
-  display::Frame*     m_frame;
-  bool                m_focus;
+  display::Frame*     m_frame{};
+  bool                m_focus{};
 
   input::Bindings     m_bindings;
   slot_type           m_slot_exit;
